package app.fedilab.fedilabtube.asynctasks;
/* Copyright 2020 Thomas Schneider
 *
 * This file is a part of TubeLab
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * TubeLab is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with TubeLab; if not,
 * see <http://www.gnu.org/licenses>. */

import android.content.Context;
import android.content.SharedPreferences;
import android.database.sqlite.SQLiteDatabase;
import android.os.AsyncTask;

import java.lang.ref.WeakReference;
import java.util.ArrayList;

import app.fedilab.fedilabtube.client.APIResponse;
import app.fedilab.fedilabtube.client.PeertubeAPI;
import app.fedilab.fedilabtube.client.entities.Account;
import app.fedilab.fedilabtube.client.entities.Playlist;
import app.fedilab.fedilabtube.helper.Helper;
import app.fedilab.fedilabtube.interfaces.OnPlaylistActionInterface;
import app.fedilab.fedilabtube.sqlite.AccountDAO;
import app.fedilab.fedilabtube.sqlite.Sqlite;


public class ManagePlaylistsAsyncTask extends AsyncTask<Void, Void, Void> {

    private OnPlaylistActionInterface listener;
    private APIResponse apiResponse;
    private int statusCode;
    private action apiAction;
    private WeakReference<Context> contextReference;
    private String max_id;
    private Playlist playlist;
    private String videoId;

    public ManagePlaylistsAsyncTask(Context context, action apiAction, Playlist playlist, String videoId, String max_id, OnPlaylistActionInterface onPlaylistActionInterface) {
        contextReference = new WeakReference<>(context);
        this.listener = onPlaylistActionInterface;
        this.apiAction = apiAction;
        this.max_id = max_id;
        this.playlist = playlist;
        this.videoId = videoId;
    }

    @Override
    protected Void doInBackground(Void... params) {
        SharedPreferences sharedpreferences = contextReference.get().getSharedPreferences(Helper.APP_PREFS, Context.MODE_PRIVATE);
        String userId = sharedpreferences.getString(Helper.PREF_KEY_ID, null);
        String instance = Helper.getLiveInstance(contextReference.get());
        SQLiteDatabase db = Sqlite.getInstance(contextReference.get().getApplicationContext(), Sqlite.DB_NAME, null, Sqlite.DB_VERSION).open();
        Account account = new AccountDAO(contextReference.get(), db).getUniqAccount(userId, instance);
        if (account == null) {
            account = new AccountDAO(contextReference.get(), db).getUniqAccount(userId, Helper.getPeertubeUrl(instance));
        }
        if (account == null) {
            statusCode = 403;
            apiResponse = new APIResponse();
            apiResponse.setPlaylists(new ArrayList<>());
        } else if (apiAction == action.GET_PLAYLIST) {
            apiResponse = new PeertubeAPI(contextReference.get()).getPlayists(account.getUsername());
        } else if (apiAction == action.GET_LIST_VIDEOS) {
            apiResponse = new PeertubeAPI(contextReference.get()).getPlaylistVideos(playlist.getId(), max_id, null);
        } else if (apiAction == action.DELETE_PLAYLIST) {
            statusCode = new PeertubeAPI(contextReference.get()).deletePlaylist(playlist.getId());
        } else if (apiAction == action.ADD_VIDEOS) {
            statusCode = new PeertubeAPI(contextReference.get()).addVideoPlaylist(playlist.getId(), videoId);
        } else if (apiAction == action.DELETE_VIDEOS) {
            statusCode = new PeertubeAPI(contextReference.get()).deleteVideoPlaylist(playlist.getId(), videoId);
        } else if (apiAction == action.GET_PLAYLIST_FOR_VIDEO) {
            apiResponse = new PeertubeAPI(contextReference.get()).getPlaylistForVideo(videoId);
        }
        return null;
    }

    @Override
    protected void onPostExecute(Void result) {
        listener.onActionDone(this.apiAction, apiResponse, statusCode);
    }

    public enum action {
        GET_PLAYLIST,
        GET_LIST_VIDEOS,
        CREATE_PLAYLIST,
        DELETE_PLAYLIST,
        ADD_VIDEOS,
        DELETE_VIDEOS,
        GET_PLAYLIST_FOR_VIDEO,
    }

}
